@@ -1,7 +1,7 @@
 <?php declare(strict_types = 0);
 /*
 ** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -248,7 +248,7 @@ class CRouter {
 		'proxy.delete'								=> ['CControllerProxyDelete',							'layout.json',			null],
 		'proxy.host.disable'						=> ['CControllerProxyHostDisable',						'layout.json',			null],
 		'proxy.host.enable'							=> ['CControllerProxyHostEnable',						'layout.json',			null],
-		'proxy.list'								=> ['CControllerProxyList',								'layout.htmlpage',		'proxy.list'],
+		'proxy.list'								=> ['CControllerProxyList',								'layout.htmlpage',		'administration.proxy.list'],
 		'proxy.update'								=> ['CControllerProxyUpdate',							'layout.json',			null],
 		'queue.details'								=> ['CControllerQueueDetails',							'layout.htmlpage',		'administration.queue.details'],
 		'queue.overview'							=> ['CControllerQueueOverview',							'layout.htmlpage',		'administration.queue.overview'],
